import requests
import ujson as json

appUrl = "http://localhost:3000/"
test_data_provider_name = "test_create_data_provider"

data_providers = []


def test_get_data_providers():
    global data_providers
    url = appUrl + "data-providers"
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 200
    print(resp.text)
    assert type(json.loads(resp.text)) is list

    for dp in json.loads(resp.text):
        assert type(dp) is dict
        assert "name" in dp
        assert "type" in dp

    data_providers = json.loads(resp.text)


def test_get_data_provider():
    for dp in data_providers:
        url = appUrl + "data-providers/" + dp["name"]
        resp = requests.get(url=url, headers={})
        assert resp.status_code == 200


def test_add_data_provider():
    url = appUrl + "data-providers"
    data = {"name": test_data_provider_name, "type": "I DONT EXIST"}
    resp = requests.post(url=url, headers={}, json=data)
    assert resp.status_code == 400


def test_add_web_data_provider():
    # Test no url
    url = appUrl + "data-providers"
    data = {"name": test_data_provider_name, "type": "Web"}
    resp = requests.post(url=url, headers={}, json=data)
    assert resp.status_code == 400

    # Test bad url
    data = {"name": test_data_provider_name, "type": "Web", "url": ""}
    resp = requests.post(url=url, headers={}, json=data)
    assert resp.status_code == 400
    data = {"name": test_data_provider_name, "type": "Web", "url": "I DONT EXIST"}
    resp = requests.post(url=url, headers={}, json=data)
    assert resp.status_code == 400

    # Test good url
    data = {
        "name": test_data_provider_name,
        "type": "Web",
        "url": "http://localhost:4000",
    }
    resp = requests.post(url=url, headers={}, json=data)
    assert resp.status_code == 204

    # Test that it exists
    url = appUrl + "data-providers/" + test_data_provider_name
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 200

    # Test that it was added to the list
    url = appUrl + "data-providers"
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 200
    assert type(json.loads(resp.text)) is list
    assert len(json.loads(resp.text)) == len(data_providers) + 1
    assert any(dp["name"] == test_data_provider_name for dp in json.loads(resp.text))


def test_delete_data_provider():
    url = appUrl + "data-providers/" + test_data_provider_name
    resp = requests.delete(url=url, headers={})
    assert resp.status_code == 204

    # Test that it was removed from the list
    url = appUrl + "data-providers"
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 200
    assert type(json.loads(resp.text)) is list
    assert len(json.loads(resp.text)) == len(data_providers)
    assert not any(
        dp["name"] == test_data_provider_name for dp in json.loads(resp.text)
    )

    # Test that it no longer exists
    url = appUrl + "data-providers/" + test_data_provider_name
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 404



from pickledb import PickleDB
from ..exploration_statistics.utils import get_data_batch, get_columns_statistics
from ..modules.dataProviders.dataProviderManager import (
    get_single_data_provider_for_project_id,
)
from uuid import uuid4
from time import time
import traceback
from collections import defaultdict

# Create or load a database
project_explorations_db = PickleDB("projectsExplorations.db")
computation_threads = {}
stop_flags = {}


# DB operations
def get_exploration_by_id(project_id, exploration_id):
    explorations = project_explorations_db.get(project_id) or []
    for exploration in explorations:
        if exploration.get("id") == exploration_id:
            return exploration
    return None


def update_explorations(project_id, explorations):
    project_explorations_db.set(project_id, explorations)
    project_explorations_db.save()


def update_exploration(project_id, exploration):
    explorations = project_explorations_db.get(project_id) or []
    for i, existing_exploration in enumerate(explorations):
        if existing_exploration.get("id") == exploration.get("id"):
            explorations[i] = exploration
            break
    else:
        explorations.append(exploration)

    update_explorations(project_id, explorations)
    return exploration


# Processing functions
def start_exploration_real_combination_computation(project_id, exploration_id):
    try:
        _start_exploration_real_combination_computation(project_id, exploration_id)
    except Exception as e:
        print(f"Error during exploration computation: {e}")
        traceback.print_exc()
        exploration = get_exploration_by_id(project_id, exploration_id)
        if exploration:
            exploration["state"] = "error"
            update_exploration(project_id, exploration)
        return


def _start_exploration_real_combination_computation(project_id, exploration_id):
    # Get the exploration from the database
    exploration = get_exploration_by_id(project_id, exploration_id)
    if exploration is None:
        raise ValueError(
            f"Exploration with ID {exploration_id} not found in project {project_id}"
        )
    selected_columns = exploration.get("config", {}).get("columns", [])
    if not selected_columns:
        raise ValueError(
            "No selected columns for exploration. Please select columns to explore."
        )
    selected_columns_labels = [col["label"] for col in selected_columns]
    selected_columns_aggregations = {
        col["label"]: col.get("aggregation", "none") for col in selected_columns
    }

    # Get the data-provider for the given project
    data_provider = get_single_data_provider_for_project_id(project_id)
    data_provider_info = data_provider.get_info()
    project = data_provider.get_project(project_id)
    columns_structure = project["columns"]
    project_nb_samples = project.get("nbSamples", None)

    # Set the exploration as started
    exploration["state"] = "ongoing"
    exploration["real_combinations"] = 0
    exploration["started_at"] = time()
    exploration["current_sample"] = 0
    exploration["remaining_time"] = None
    update_exploration(project_id, exploration)

    # Load the columns statistics
    columns_statistics = get_columns_statistics(data_provider.name, project_id)[
        "columns"
    ]
    columns_statistics_dict = {col["name"]: col for col in columns_statistics}

    # Set the metrics up
    metrics_values = {"Nb Samples": {"values": []}}
    metric_columns_to_fetch = set()
    for selectedSampleMetric in exploration.get("config", {}).get(
        "selectedSampleMetrics", []
    ):
        metrics_values[selectedSampleMetric] = {"values": []}
    for selectedColumnWithMetrics in exploration.get("config", {}).get(
        "column_metrics", []
    ):
        for selectedColumnMetric in selectedColumnWithMetrics.get("metrics", []):
            metric_name = (
                selectedColumnWithMetrics["columnLabel"] + "_" + selectedColumnMetric
            )
            metrics_values[metric_name] = {
                "values": [],
                "metric": selectedColumnMetric,
                "column": selectedColumnWithMetrics["columnLabel"],
            }
            metric_columns_to_fetch.add(selectedColumnWithMetrics["columnLabel"])
    metric_columns_to_fetch = list(metric_columns_to_fetch)

    # Determine the number of samples to fetch in each batch
    NB_SAMPLES = 25000
    if data_provider_info.get("maxSampleIdByRequest") and data_provider_info.get(
        "maxSampleDataByRequest"
    ):
        NB_SAMPLES = min(
            data_provider_info["maxSampleIdByRequest"],
            data_provider_info["maxSampleDataByRequest"],
        )

    # Combinations computation
    combinations = defaultdict(list)
    current_sample = 0
    computation_id = str(uuid4())
    while True:
        # Check if the stop signal is set, stop the exploration if it is
        if stop_flags.get(exploration_id):
            print(f"Stopping exploration {exploration_id}")
            exploration["state"] = "not_started"
            update_exploration(project_id, exploration)
            return

        # Fetch the next batch of samples
        batch = get_data_batch(
            data_provider,
            project_id,
            {"id": computation_id},
            current_sample,
            current_sample + NB_SAMPLES - 1,
            columns_structure,
            selected_columns_labels,
            selected_columns_aggregations,
            columns_statistics_dict,
            metric_columns_to_fetch,
        )

        # Update the combinations
        for data_id, values in batch.items():
            combinations[values["column_values"]].append(data_id)

        # Process the metrics for the current batch
        process_combination_metrics(
            current_combinations=combinations,
            data_and_metrics_batch=batch,
            metrics=metrics_values,
        )

        # Update the exploration progression status
        current_sample += len(batch)
        exploration["current_sample"] = current_sample
        exploration["real_combinations"] = len(combinations.keys())

        # Calculate the estimated time remaining
        elapsed_time = time() - exploration["started_at"]
        if exploration["current_sample"] > 0 and project_nb_samples is not None:
            estimated_total_time = (
                elapsed_time * project_nb_samples / exploration["current_sample"]
            )
            exploration["remaining_time"] = max(0, estimated_total_time - elapsed_time)

        # Update the exploration in the database
        update_exploration(project_id, exploration)

        # If the batch is less than the requested size, we assume we reached the end
        if len(batch) < NB_SAMPLES:
            print("Reached the end of the data provider samples.")
            break

    # Convert combinations to JSON format
    combinations_json = []
    for key, value in combinations.items():
        combinations_json.append({"sample_ids": value, "combination": list(key)})

    # Clean the metrics values
    for metric_name, metric_data in metrics_values.items():
        keys = list(metric_data.keys())
        for key in keys:
            if key != "values":
                del metric_data[key]

    # Set the exploration status to "completed"
    exploration["combinations"] = combinations_json
    exploration["metrics"] = metrics_values
    exploration["state"] = "completed"
    exploration["finished_at"] = time()

    update_exploration(project_id, exploration)


def process_combination_metrics(current_combinations, data_and_metrics_batch, metrics):
    """
    current_combinations:
    {
        (value1, value2): [sample_id1, sample_id2],
        (value3, value4): [sample_id3]
    }

    data_and_metrics_batch:
    {
        "dataId1": {
            "column_values": [1, "A"],
            "metric_column_values": {
                "col1": 0.5,
                "col2": 1.0,
            },
        },
        "dataId2": {
            "column_values": [2, "B"],
            "metric_column_values": {
                "col1": 0.7,
                "col2": 1.2,
            },
        },
        ...
    }

    metrics:
    {
        "Nb Samples": {
            "values": [...],
        },
        "metric1": {
            "values": [...],
            "column": "column1",
            "metric": "mean",
            ...
        },
        ...
    """

    if "Nb Samples" in metrics:
        # For each combination, count the number of samples
        metrics["Nb Samples"]["values"] = []
        for combination in current_combinations:
            metrics["Nb Samples"]["values"].append(
                len(current_combinations[combination])
            )

    for metric_name, metric_data in metrics.items():
        column_name = metric_data.get("column")
        if metric_data.get("metric") == "mean":
            # Process mean metrics
            if "combinations" not in metric_data:
                metric_data["combinations"] = {}

            for new_data in data_and_metrics_batch.values():
                if new_data["column_values"] not in metric_data["combinations"]:
                    metric_data["combinations"][new_data["column_values"]] = {
                        "mean": 0,
                        "count": 0,
                    }

                current_mean = metric_data["combinations"][new_data["column_values"]][
                    "mean"
                ]
                metric_data["combinations"][new_data["column_values"]]["count"] += 1
                current_count = metric_data["combinations"][new_data["column_values"]][
                    "count"
                ]

                metric_data["combinations"][new_data["column_values"]]["mean"] += (
                    new_data["metric_column_values"].get(column_name, 0) - current_mean
                ) / (current_count)

            # Save the mean for each combination
            metric_data["values"] = []
            for combination in current_combinations:
                if combination in metric_data["combinations"]:
                    mean_value = metric_data["combinations"][combination].get("mean", 0)
                    metric_data["values"].append(mean_value)
                else:
                    metric_data["values"].append(0)


# Selections
def create_selection(
    project_id, exploration_id, selected_combinations_id, selection_name
):
    exploration = get_exploration_by_id(project_id, exploration_id)
    if exploration is None:
        raise ValueError(
            f"Exploration with ID {exploration_id} not found in project {project_id}"
        )

    # Get the data provider for the project
    data_provider = get_single_data_provider_for_project_id(project_id)
    if data_provider is None:
        raise ValueError(f"No data provider found for project {project_id}")

    # Extract the selected combinations from the exploration
    combinations = exploration.get("combinations", [])
    if not combinations:
        raise ValueError(
            f"No combinations found in exploration \
{exploration_id} for project {project_id}"
        )

    # Filter the combinations based on the selected IDs
    selected_combinations = [
        comb for comb in combinations if comb["combination"] in selected_combinations_id
    ]

    # Aggregate the sample IDs from the selected combinations
    selected_sample_ids = []
    for comb in selected_combinations:
        selected_sample_ids.extend(comb["sample_ids"])

    # Create the selection
    data_provider.create_selection(
        project_id,
        selection_name,
        selected_sample_ids,
    )



#############################################################################
# Imports
#############################################################################
from debiaiServer.modules.dataProviders.DataProviderException import (
    DataProviderException,
)
import debiaiServer.modules.dataProviders.dataProviderManager as data_provider_manager

#############################################################################
# PROJECTS Management
#############################################################################


def ping():
    return "Online", 200


def get_data_providers_project(dataProviderId):
    # Return a list of project overviews for a specific data provider
    data_provider = data_provider_manager.get_single_data_provider(dataProviderId)

    if data_provider is None:
        return "Data provider not found", 404

    try:
        projects = data_provider.get_projects()

        if projects is not None:
            # Adding data provider id to projects
            for project in projects:
                project["dataProviderId"] = data_provider.name

    except DataProviderException as e:
        print("Warning get DP projects : " + e.message)

    return projects, 200


def get_projects():
    # Return a list of project overviews from all the data providers
    data_providers_list = data_provider_manager.get_data_provider_list()
    projectOverviews = []
    for data_provider in data_providers_list:
        try:
            projects = data_provider.get_projects()

            if projects is not None:
                # Adding data provider id to projects
                for project in projects:
                    project["dataProviderId"] = data_provider.name

                projectOverviews.extend(projects)

        except DataProviderException as e:
            print("Warning get DP projects : " + e.message)

    return projectOverviews, 200


def get_project(dataProviderId, projectId):
    # return the info about datasets, models, selections & tags
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)

        project = data_provider.get_project(projectId)

        # Adding data provider id to project
        project["dataProviderId"] = dataProviderId

        return project, 200
    except DataProviderException as e:
        return e.message, e.status_code


def get_data_id_list(dataProviderId, projectId, requestParameters):
    # return the list of data ids
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)

        data_id_list = data_provider.get_id_list(
            projectId,
            requestParameters["analysis"],
            requestParameters["from"],
            requestParameters["to"],
        )

        return data_id_list, 200
    except DataProviderException as e:
        return e.message, e.status_code


def delete_project(dataProviderId, projectId):
    # Delete a project
    try:
        data_provider = data_provider_manager.get_single_data_provider(dataProviderId)

        data_provider.delete_project(projectId)
        return "Project deleted", 200
    except DataProviderException as e:
        return e.message, e.status_code



import os
import sys
import errno
import argparse
import webbrowser
from threading import Timer
from termcolor import colored
from debiaiServer.config.init_config import SUCCESS_COLOR
from debiaiServer.utils.utils import Loader

sys.stdout.reconfigure(encoding="utf-8")

DEFAULT_PORT = 3000
DEFAULT_DATA_FOLDER = "debiai_data"

welcome_logo = """
                                                    █████████████            
                                                 ███████████████████         
                                               ███████████████████████       
                                             ██████████████████████████      
   █████████             ████        ███    ██████████   ██████   ███████    
    ███    ███            ███              ██████████     █████   ████████   
    ███     ███  ███████  ████████  ████   █████████   █   ████   ████████   
    ███     ███ ███   ███ ███   ███  ███   ████████   ███   ███   ████████   
    ███     ███ ████████  ███   ███  ███   ███████           ██   ████████   
    ███    ███  ███       ███   ███  ███   ███████   █████   ██   ████████   
   ██████████    ███████  ████████  █████  ██████   ███████   █   ███████    
                                            █████████████████████████████    
                                             ██████████████████████████      
                                            ██████████████████████████       
                                           █████████████████████████         
                                         ██████     █████████████            
                                       ██████                                
                                      ████                                   
"""


def create_folder(path=None):
    """Creates a folder at the specified path."""
    path = path or os.getcwd()

    # Return the absolute path if the folder already exists
    if os.path.exists(path):
        if not os.path.isdir(path):
            raise OSError(
                errno.ENOTDIR,
                f"'{path}' is not a folder. Please provide a valid folder path.",
            )
        return os.path.abspath(path)

    # Create the folder if it does not exist
    create_folder_answer = gather_user_input(
        f"The folder {path} does not exist. Create it? (Y/n): "
    )
    if create_folder_answer.lower() in ("y", ""):
        os.makedirs(path)
        print(f"DebiAI data folder '{path}' created successfully.")
    else:
        print("Folder creation cancelled, exiting.")
        exit()

    return os.path.abspath(path)


def gather_user_input(message):
    try:
        return input(message)
    except KeyboardInterrupt:
        print("\nGoodbye!")
        exit()


def setup_data_folder(data_folder_path: str = None):
    if data_folder_path:
        created_data_folder_path = create_folder(data_folder_path)
    else:
        print("DebiAI requires a folder to store the data.")
        created_data_folder_path = create_folder(
            gather_user_input(f"Enter data folder path ({DEFAULT_DATA_FOLDER}): ")
            or DEFAULT_DATA_FOLDER
        )
        print("use --data-folder to specify the data folder path")
    print("Data folder path:", created_data_folder_path, "\n")
    return created_data_folder_path


def open_browser(port):
    # Check if the browser is open
    is_browser_open = False
    try:
        webbrowser.get()
        is_browser_open = True
    except webbrowser.Error:
        pass

    # Open the browser
    try:
        url = f"http://localhost:{port}"
        if is_browser_open:
            webbrowser.open_new_tab(url)
        else:
            webbrowser.open(url)
    except webbrowser.Error:
        print(
            colored(
                "Unable to open the browser. Please open your browser and go to " + url,
                "red",
            )
        )


def debiai_gui_start(path, port, no_browser=False, parameters: dict = {}):
    # Display welcome message
    print(welcome_logo)
    print("   Welcome!\n")

    # Ask for the data folder path
    data_folder_path = setup_data_folder(path)

    # Import the DebiAI GUI with a loading animation
    start_loading_animation = Loader("Starting the DebiAI GUI", "")
    start_loading_animation.start()
    from debiaiServer.websrv import start_server

    start_loading_animation.stop()

    # Open the browser with a little delay
    if not no_browser:
        Timer(0.5, open_browser, args=[port]).start()

    # Start the DebiAI GUI
    start_server(
        port=port,
        data_folder_path=data_folder_path,
        parameters=parameters,
        reloader=False,
        is_dev=False,
    )


def build_parameters(args):
    # Transform the command line arguments into a config compatible dictionary
    # Similar to the one used in the config file and environment variables
    parameters = {}
    if args.data_provider and type(args.data_provider) is list:
        for i, data_provider in enumerate(args.data_provider):
            if data_provider:
                dp_name = f"DataProvider-{i}"
                parameters[f"DEBIAI_WEB_DATA_PROVIDER_{dp_name}"] = data_provider
    if args.algo_provider and type(args.algo_provider) is list:
        for i, algo_provider in enumerate(args.algo_provider):
            if algo_provider:
                ap_name = f"AlgoProvider-{i}"
                parameters[f"DEBIAI_ALGO_PROVIDER_{ap_name}"] = algo_provider

    return parameters


def main():
    parser = argparse.ArgumentParser(
        description="""DebiAI GUI Command Line.
Learn more about DebiAI at https://debiai.irt-systemx.fr/"""
    )

    # Version flag
    parser.add_argument(
        "-v", "--version", action="store_true", help="Print DebiAI version number"
    )

    # Commands
    subparsers = parser.add_subparsers(
        dest="command",
        title="DebiAI-GUI",
        help="Available commands",
        metavar="[command]",
    )

    # Start command
    start_parser = subparsers.add_parser("start", help="Start the DebiAI GUI")
    start_parser.add_argument(
        "-d",
        "--data-folder",
        type=str,
        help="Path to the data folder",
    )
    start_parser.add_argument(
        "-p",
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help="Port to run the server on",
    )
    start_parser.add_argument(
        "-nb",
        "--no-browser",
        action="store_true",
        help="Do not open the browser on startup",
    )
    start_parser.add_argument(
        "-dp",
        "--data-provider",
        type=str,
        nargs="+",  # 1 or more arguments
        help="One or more data-provider URLs",
    )
    start_parser.add_argument(
        "-ap",
        "--algo-provider",
        type=str,
        nargs="+",  # 1 or more arguments
        help="One or more algo-provider URLs",
    )

    # Parse the arguments
    args = parser.parse_args()
    if args.version:
        from debiaiServer.utils.utils import get_app_version

        print("DebiAI Version:", colored(get_app_version(), SUCCESS_COLOR))
    elif args.command == "start":
        debiai_gui_start(
            path=args.data_folder,
            port=args.port,
            no_browser=args.no_browser,
            parameters=build_parameters(args),
        )
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



#############################################################################
# Imports
#############################################################################
from debiaiServer.config.init_config import get_config
from debiaiServer.utils.utils import is_url_valid, is_valid_name
import debiaiServer.modules.algoProviders.algoProvidersManager as algo_provider_manager
from debiaiServer.modules.algoProviders.AlgoProviderException import (
    AlgoProviderException,
)
from debiaiServer.modules.algoProviders.AlgoProvider import AlgoProvider

#############################################################################
# Algo providers Management
#############################################################################


def get_algo_providers():
    algorithms = algo_provider_manager.get_algo_providers_json()
    return algorithms, 200


def post_algo_provider(data):
    # Check if we are allowed to add AlgoProviders from the config file
    config = get_config()
    creation_allowed = config["ALGO_PROVIDERS_CONFIG"]["creation"]
    if not creation_allowed:
        return "AlgoProvider creation is not allowed", 403

    # Check if algoProviders already exists
    if algo_provider_manager.algo_provider_exists(data["name"]):
        return "AlgoProvider '" + data["name"] + "' already exists", 400

    # Check if algoProviders name is valid
    if not is_valid_name(data["name"]):
        return "Invalid algoProviders name", 400

    # Add the algoProvider
    # Check if url is valid
    if not is_url_valid(data["url"]):
        return "Invalid url", 400

    algo_provider_manager.add(AlgoProvider(data["url"], data["name"]))

    return None, 204


def use_algo(algoProviderName, algoId, data):
    # Check if algoProviders exists
    if not algo_provider_manager.algo_provider_exists(algoProviderName):
        return "AlgoProvider " + algoProviderName + " does not exists", 404

    try:
        # Use algoProviders
        algo_provider = algo_provider_manager.get_single_algo_provider(algoProviderName)
        return algo_provider.use_algorithm(algoId, data), 200
    except AlgoProviderException as e:
        return e.message, e.status_code


def delete_algo_provider(name):
    # Check if we are allowed to add AlgoProviders from the config file
    config = get_config()
    deletion_allowed = config["ALGO_PROVIDERS_CONFIG"]["deletion"]
    if not deletion_allowed:
        return "AlgoProvider deletion is not allowed", 403

    # Delete the algoProvider
    try:
        algo_provider_manager.delete(name)
        return None, 204
    except AlgoProviderException as e:
        return e.message, e.status_code



from debiaiServer.modules.exportMethods.exportClass import ExportType, ExportMethod
import requests

#############################################################################
#
# HTTP - Post export method
#
# Send data through a post request to a server
#
#############################################################################


class PostExportType(ExportType):
    def __init__(self):
        super().__init__()

        self.name = "post"
        self.parameters_definition = ["url"]

        self.export_method_class = PostExportMethod


class PostExportMethod(ExportMethod):
    up = False

    def __init__(self, name, parameters):
        super().__init__(PostExportType(), name, parameters)

        # Expected parameters: [url]
        # Check parameters
        if len(parameters) != 1:
            raise Exception("Post export type requires 1 parameter : the url")

        self.url = parameters[0]

        # Check url
        if not self.url.startswith("http://") and not self.url.startswith("https://"):
            raise Exception(
                "Url '" + self.url + "' must start with http:// or https://"
            )

        self.up = True

    def export(self, data):
        print("Post export method: Sending data to '" + self.url + "'")

        if not self.up:
            raise Exception("Can't send data to '" + self.url + "'")

        try:
            # Send data
            r = requests.post(self.url, json=data)
            r.raise_for_status()
            print("Post export method : Data sent")
        except Exception as e:
            print("Post export method : Error sending post request", e)
            raise Exception(
                "Post export method : Error sending post request on url" + str(e)
            )



import requests
import ujson as json

PYTHON_DATA_PROVIDER_ID = "Python module Data Provider"
appUrl = "http://localhost:3000/"
test_project_name = "test_create_project"
test_project_id = None

# ============== PROJECTS =================


def test_get_projects():
    url = appUrl + "projects"
    resp = requests.get(url=url, headers={})
    assert resp.status_code == 200
    assert type(json.loads(resp.text)) is list


def test_get_bad_project():
    projectId = "I_DO_NOT_EXIST"
    url = (
        appUrl + "data-providers/" + PYTHON_DATA_PROVIDER_ID + "/projects/" + projectId
    )
    resp = requests.request("GET", url, headers={}, data={})
    assert resp.status_code == 404

    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 404


def test_create_project_noName():
    # create
    url = appUrl + "projects"
    resp = requests.post(url=url, headers={}, json={})
    assert resp.status_code == 400


def test_create_project():
    global test_project_id
    # delete if exists
    projectId = test_project_name
    url = (
        appUrl + "data-providers/" + PYTHON_DATA_PROVIDER_ID + "/projects/" + projectId
    )
    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 200 or resp.status_code == 404

    # create
    url = appUrl + "projects"
    resp = requests.post(url=url, headers={}, json={"projectName": test_project_name})
    assert resp.status_code == 200

    # Get Id
    data = json.loads(resp.text)
    test_project_id = data["id"]
    assert test_project_id is not None
    assert len(test_project_id) > 0
    assert type(test_project_id) is str

    # Test can't create same project
    resp = requests.post(url=url, headers={}, json={"projectName": test_project_name})
    assert resp.status_code == 400
    assert "already exists" in resp.text


def test_get_project():
    # Find back
    url = (
        appUrl
        + "data-providers/"
        + PYTHON_DATA_PROVIDER_ID
        + "/projects/"
        + test_project_id
    )
    resp = requests.request("GET", url, headers={}, json={})
    assert resp.status_code == 200
    proj = json.loads(resp.text)
    assert type(proj) is dict
    assert type(proj["columns"]) is list
    assert proj["models"] == []
    assert len(proj["name"]) > 0
    assert proj["name"] == test_project_name


def test_remove_project():
    # Project exists back
    url = (
        appUrl
        + "data-providers/"
        + PYTHON_DATA_PROVIDER_ID
        + "/projects/"
        + test_project_id
    )
    resp = requests.request("GET", url, headers={}, json={})
    assert resp.status_code == 200

    # remove
    url = (
        appUrl
        + "data-providers/"
        + PYTHON_DATA_PROVIDER_ID
        + "/projects/"
        + test_project_id
    )
    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 200

    # Dont Find back
    url = (
        appUrl
        + "data-providers/"
        + PYTHON_DATA_PROVIDER_ID
        + "/projects/"
        + test_project_id
    )
    resp = requests.request("GET", url, headers={}, json={})
    assert resp.status_code == 404

    # Cant remove again
    url = (
        appUrl
        + "data-providers/"
        + PYTHON_DATA_PROVIDER_ID
        + "/projects/"
        + test_project_id
    )
    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 404


def test_project_nameTooLong():
    testProjectName = "a" * 256
    url = appUrl + "projects"
    payload = {"projectName": testProjectName, "blockLevelInfo": []}
    headers = {"content-type": "application/json"}
    resp = requests.post(url=url, headers=headers, json=payload)
    assert resp.status_code == 400



import requests
import ujson as json

appUrl = "http://localhost:3000/"
testWidgetKey = "testWidgetKey"
testConfigurationId = None


def delete_configuration(id):
    url = appUrl + "app/widgets/" + testWidgetKey + "/configurations/" + id
    resp = requests.request("DELETE", url, headers={}, data={})
    assert resp.status_code == 204


def test_get_configurations():
    url = appUrl + "app/widgets/" + testWidgetKey + "/configurations"
    resp = requests.request("GET", url, headers={}, data={})
    configurations = json.loads(resp.text)
    print(configurations)
    assert resp.status_code == 200
    assert type(configurations) is list

    # Remove all configurations
    for conf in configurations:
        assert type(conf) is dict
        assert "id" in conf
        delete_configuration(conf["id"])


def test_add_configuration():
    global testConfigurationId
    url = appUrl + "app/widgets/" + testWidgetKey + "/configurations"
    data = {
        "name": "testName",
        "description": "testDescription",
        "projectId": "testProjectId",
        "dataProviderId": "testDataProviderId",
        "configuration": {"testKey": "testValue"},
    }
    resp = requests.request("POST", url, headers={}, json=data)
    assert resp.status_code == 204

    # Check if the configuration was added
    resp = requests.request("GET", url, headers={}, data={})
    configurations = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(configurations) is list
    assert len(configurations) == 1
    assert configurations[0]["name"] == data["name"]
    assert configurations[0]["description"] == data["description"]
    assert configurations[0]["projectId"] == data["projectId"]
    assert configurations[0]["dataProviderId"] == data["dataProviderId"]
    assert type(configurations[0]["configuration"]) is dict
    assert (
        configurations[0]["configuration"]["testKey"]
        == data["configuration"]["testKey"]
    )

    assert "id" in configurations[0]
    assert type(configurations[0]["id"]) is str
    assert len(configurations[0]["id"]) > 0
    testConfigurationId = configurations[0]["id"]


def test_get_configurations_overview():
    url = appUrl + "app/widget-configurations"
    resp = requests.request("GET", url, headers={}, data={})
    configurations = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(configurations) is dict
    assert testWidgetKey in configurations
    assert configurations[testWidgetKey] == 1


def test_delete_configuration():
    # Remove the configuration
    delete_configuration(testConfigurationId)

    # Check if the configuration was removed
    url = appUrl + "app/widgets/" + testWidgetKey + "/configurations"
    resp = requests.request("GET", url, headers={}, data={})
    configurations = json.loads(resp.text)
    assert resp.status_code == 200
    assert type(configurations) is list
    assert len(configurations) == 0



import debiaiServer.modules.dataProviders.webDataProvider.http.api as api

from debiaiServer.modules.dataProviders.webDataProvider.useCases.models import (
    get_models_info,
)
from debiaiServer.modules.dataProviders.webDataProvider.useCases.selections import (
    get_project_selections,
)


def get_all_projects_from_data_provider(url, name):
    projects = api.get_projects(url)
    project_list = []

    if not projects:
        return

    for project_id in projects:
        if "nbSamples" not in projects[project_id]:
            projects[project_id]["nbSamples"] = None

        if "nbModels" not in projects[project_id]:
            projects[project_id]["nbModels"] = None

        if "nbSelections" not in projects[project_id]:
            projects[project_id]["nbSelections"] = None

        if "name" not in projects[project_id]:
            projects[project_id]["name"] = project_id

        if "creationDate" not in projects[project_id]:
            projects[project_id]["creationDate"] = None

        if "updateDate" not in projects[project_id]:
            projects[project_id]["updateDate"] = None

        project_list.append(
            {
                "id": project_id,
                "dataProvider": name,
                "name": projects[project_id]["name"],
                "nbModels": projects[project_id]["nbModels"],
                "nbSamples": projects[project_id]["nbSamples"],
                "nbSelections": projects[project_id]["nbSelections"],
                "creationDate": projects[project_id]["creationDate"],
                "updateDate": projects[project_id]["updateDate"],
            }
        )

    return project_list


def get_single_project_from_data_provider(url, data_provider_name, id_project):
    project = api.get_project(url, id_project)

    # Check the project columns
    project_columns = get_project_columns(project)

    # Add selections
    selections = get_project_selections(url, id_project)

    # Add models
    models = get_models_info(url, id_project)

    # Check nbSamples
    if "nbSamples" in project:
        nbSamples = project["nbSamples"]
    else:
        nbSamples = None

    # Check creationDate
    if "creationDate" in project:
        creationDate = project["creationDate"]
    else:
        creationDate = None

    # Check updateDate
    if "updateDate" in project:
        updateDate = project["updateDate"]
    else:
        updateDate = None

    # Converting views to DebiAI projects
    return {
        "id": id_project,
        "name": project["name"] if "name" in project else id_project,
        "dataProvider": data_provider_name,
        "columns": project_columns,
        "resultStructure": project["expectedResults"],
        "nbModels": len(models),
        "nbSamples": nbSamples,
        "nbSelections": len(selections),
        "creationDate": creationDate,
        "updateDate": updateDate,
        "selections": selections,
        "models": models,
    }


def get_project_columns(project):
    if not project:
        return []

    # Expected project["columns"] example :
    # [
    #     { "name": "storage", "category": "other" },
    #     { "name": "age", "category": "context" },
    #     { "name": "path", "category": "input", group: "image" },
    #     { "name": "label", "category": "groundtruth", group: "image" },
    #     { "name": "type" }, # category is not specified, it will be "other"
    # ]

    project_columns = []

    if "columns" in project:
        for column in project["columns"]:
            col = {"name": column["name"]}

            if "category" in column:
                col["category"] = column["category"]
            else:
                col["category"] = "other"

            if "type" in column:
                col["type"] = column["type"]
            else:
                col["type"] = "auto"

            if "group" in column:
                col["group"] = column["group"]

            project_columns.append(col)

    return project_columns


def delete_project(url, project_id):
    api.delete_project(url, project_id)



from debiaiServer.modules.algoProviders.integratedAlgoProvider.utils import (
    get_input_from_inputs,
)


# This algorithm is a simple regression metric calculator
# It takes a list of numbers corresponding to an error
# and a ceil corresponding to the maximum acceptable error

# It returns a list of True/False values corresponding to
# whether the error is acceptable or not

# It also returns the percentage of acceptable errors

# Technical details (must respect the algo-api format):
algorithm_description = {
    "name": "Regression Metric",
    "description": """Calculates the regression error according to the ground truth, \
the predictions and a ceil value""",
    "author": "DebiAI",
    "version": "1.0.0",
    "creationDate": "2023-05-23",
    "tags": ["metrics", "regression"],
    "inputs": [
        {
            "name": "Ground truth",
            "description": "List of ground truth values",
            "type": "array",
            "arrayType": "number",
        },
        {
            "name": "Predictions",
            "description": "List of predictions, must have the same length as the  \
ground truth list",
            "type": "array",
            "arrayType": "number",
        },
        {
            "name": "Ceil",
            "description": "Maximum acceptable error, depends on the use case, >= 0",
            "type": "number",
            "default": 5,
            "availableValues": [0.1, 5, 100, 10000],
            "min": 0,
        },
    ],
    "outputs": [
        {
            "name": "Error",
            "description": "Difference between the ground truth and the predictions",
            "type": "array",
            "arrayType": "number",
        },
        {
            "name": "Absolute error",
            "description": "Absolute value of the error",
            "type": "array",
            "arrayType": "number",
        },
        {
            "name": "Binary error",
            "description": "True if Absolute error > ceil, False otherwise",
            "type": "array",
            "arrayType": "boolean",
        },
        {
            "name": "Error percentage",
            "type": "number",
        },
        {
            "name": "Binary success",
            "description": "True if Absolute error <= ceil, False otherwise",
            "type": "array",
            "arrayType": "boolean",
        },
        {
            "name": "Success percentage",
            "type": "number",
        },
    ],
}


def get_algorithm_details():
    return algorithm_description


def use_algorithm(inputs):
    # Get inputs
    gdt = get_input_from_inputs(inputs, "Ground truth", "array", "number")
    predictions = get_input_from_inputs(inputs, "Predictions", "array", "number")
    ceil = get_input_from_inputs(inputs, "Ceil", "number")

    # Check inputs
    if ceil < 0:
        raise TypeError("Ceil must be positive")

    if len(gdt) != len(predictions):
        raise TypeError("Ground truth and predictions must have the same length")

    # Calculate regression metric
    nb_values = len(gdt)
    error = [None] * nb_values
    absolute_error = [None] * nb_values
    binary_error = [None] * nb_values
    binary_success = [None] * nb_values

    for i in range(nb_values):
        error_value = gdt[i] - predictions[i]
        error[i] = error_value
        absolute_error[i] = abs(error_value)

        if abs(error_value) > ceil:
            binary_error[i] = True
            binary_success[i] = False
        else:
            binary_error[i] = False
            binary_success[i] = True

    # Calculate percentages
    error_percentage = binary_error.count(True) / nb_values
    error_percentage = round(error_percentage * 100, 2)
    success_percentage = binary_success.count(True) / nb_values
    success_percentage = round(success_percentage * 100, 2)

    # Return outputs
    return [
        {
            "name": "Error",
            "value": error,
        },
        {
            "name": "Absolute error",
            "value": absolute_error,
        },
        {
            "name": "Binary error",
            "value": binary_error,
        },
        {
            "name": "Error percentage",
            "value": error_percentage,
        },
        {
            "name": "Binary success",
            "value": binary_success,
        },
        {
            "name": "Success percentage",
            "value": success_percentage,
        },
    ]



from abc import ABC, abstractmethod


class DataProvider(ABC):
    # Data
    @property
    @abstractmethod
    def name(self):
        pass

    @property
    @abstractmethod
    def is_alive(self):
        return False

    @property
    @abstractmethod
    def type(self):
        pass

    # Info
    @abstractmethod
    def get_info(self) -> dict:
        pass

    # Projects
    @abstractmethod
    def get_projects(self):
        pass

    @abstractmethod
    def get_project(self, id):
        pass

    @abstractmethod
    def delete_project(self, _id):
        pass

    # Samples
    @abstractmethod
    def get_id_list(self, _projectId, _analysis, _from, _to):
        pass

    @abstractmethod
    def get_samples(self, _projectId, _analysis, id_list):
        pass

    # Selections
    @abstractmethod
    def get_selections(self):
        pass

    @abstractmethod
    def get_selection_id_list(self, id):
        pass

    @abstractmethod
    def create_selection(self, name, id_list):
        pass

    @abstractmethod
    def delete_selection(self, id):
        pass

    # Models
    @abstractmethod
    def get_models(self):
        pass

    @abstractmethod
    def get_model_results_id_list(self):
        pass

    @abstractmethod
    def get_model_results(self, id_list):
        pass

    @abstractmethod
    def delete_model(self, id):
        pass

    def supports_project_id(self, project_id):
        """
        Check if the data provider supports the given project ID.
        This can be overridden by specific data providers if needed.

        This function was made for the exploration compatibility.
        Avoid using it in other contexts unless necessary.
        """
        projects = self.get_projects()
        if not projects:
            return False
        return any(project["id"] == project_id for project in projects)
